##############################################################################
#
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
#
# Copyright (c) Digital Creations.  All rights reserved.
#
# This license has been certified as Open Source(tm).
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
#
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
#
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
#
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
#
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
#
#
# Disclaimer
#
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
#
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
#
##############################################################################
#
# Copyright (c) 2000 Andy McKay
# All rights reserved.
#
# Distributed under the Zope Public License (above)
#
##############################################################################

__version__ = '0.1.1'

from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import OFS.PropertyManager

import re
import os
import string
import Globals # so I can get Globals data_dir
import time
import urllib
import xmllib

manage_addZPMForm = HTMLFile('ZPM', globals())

def manage_addZPM(self, id='', title='', REQUEST=None):
    ''' Adds a new object '''
    if id == '':    id = str(self.counter())
    obj = ZPM(id, title)
    self._setObject(id, obj)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)
    else:
        return id

class ZPM(OFS.SimpleItem.Item,  Persistent, Acquisition.Implicit, AccessControl.Role.RoleManager):
    meta_type = 'ZPM'
    version = '0.2'

    ZPMStoreURL = 'http://www.zopezen.org/products/check'
    check_html = HTMLFile('ZPMCheck', globals())
    index_html = HTMLFile('ZPMIndex', globals())

    _properties = ( {'id':'ZPMStoreURL', 'type': 'string', 'mode': 'w'}, )
    manage_options = ( {'label':'View', 'action':'index_html',}, {'label':'Check Products', 'action':'check_html',}, ) + OFS.SimpleItem.Item.manage_options

    __ac_permissions__=(
        ('View', ('list_products', 'check_products', 'ignore_list', 'get_version') ),
        ('View management screens', ('manage_tabs', 'manage_main',) ),
        ('Change permissions', ('manage_access',) ), )


    def list_products(self):
        '''
        Returns a list of products in this installation, minus ignored products
        '''
        l = []
        i = self.ignore_list()
        for o in self.Control_Panel.Products._objects:
            if o['id'] not in i:
                l.append(o['id'])

        return l


    def check_products(self):
        '''
        Gets info and parses it
        '''
        lst = self.list_products()
        if len(lst) < 1: # dont bother checking when we have a standard installation
            return

        lst_string = string.join(lst, '|')
        lines = urllib.urlopen(self.ZPMStoreURL, data='Products=%s' % lst_string).readlines()

        p = ZopeProductXML()
        for line in lines:
            p.feed(string.strip(line))

        for product in p.list:
            product.local_version = self.get_product_version(product.id)
            product.new_version = 0
            try:
                v = getattr(product, 'version')
                if self._new_version(product.local_version, v, product.id):
                    product.new_version = 1
            except:
                    pass
        return p.list

    def ignore_list(self):
        '''
        List of product names to ignore, these are built in products
        that dont exist as seperate installations. This will probably grow in size....
        any may have to do some complicated checking to see what version we are.
        '''
        version = self.get_version()
        if version > 'Zope 2.2.9':
            list = ['ExternalMethod', 'MIMETools', 'MailHost', 'OFSP', 'PythonScripts', 'SiteAccess', 'StandardCacheManagers', 'ZCatalog', 'ZGadflyDA', 'ZSQLMethods', 'ZopeTutorial'] # Zope 2.3
        else:
            list = ['ExternalMethod', 'MIMETools', 'MailHost', 'OFSP', 'ZCatalog', 'ZGadflyDA', 'ZSQLMethods', 'ZopeTutorial', 'dtmlHelp'] # Zope 2.2.1

        return list

    def get_product_version(self, product):
        ''' Product version '''
        p = getattr(self.Control_Panel.Products, product)
        return p.version

    def get_version(self):
        ''' Zope version '''
        return self.Control_Panel.version_txt()

    def _new_version(self, local_version, version, title):
        '''
        Tries to compare the two versions, ok its a hack, but hey it works
        this will obviously be replaced by a cleaner regex soon
        '''
        title = string.lower(title)
        local_version = string.lower(local_version)
        local_version = re.sub('[.-]', '', local_version)
        local_version = re.sub(title, '', local_version)

        version = string.lower(version)
        version = re.sub('[.-]', '', version)
        version = re.sub(title, '', version)
        if local_version != '' and version != '':
            if string.strip(local_version) < string.strip(version): return 1


class ZopeProductForUser:
    def __init__(self, id, d):
        self.id = id
        for key in d.keys():
            setattr(self, key, d[key])

class ZopeProductXML(xmllib.XMLParser):
    '''
    Parse a ZopeProductXML file
    '''

    # init, set up elements
    def __init__(self):
        xmllib.XMLParser.__init__(self)
        self.info = {}
        self.list = []
        self.lasttag = ''
        self.elements = {
            'id': (self.start_tag, self.end_tag),
            'title': (self.start_tag, self.end_tag),
            'author': (self.start_tag, self.end_tag),
            'date': (self.start_tag, self.end_tag),
            'url': (self.start_tag, self.end_tag),
            'error': (self.start_tag, self.end_tag),
            'version': (self.start_tag, self.end_tag),
            'product': (self.start_product, self.end_product)
        }

    # specific event on product
    def start_product(self, tag):
        self.info = {}

    def end_product(self):
        self.list.append(ZopeProductForUser(self.info['id'], self.info))
        self.info = {}

    # general event for rest
    def start_tag(self, tag):
        self.lasttag = tag

    def end_tag(self):
        pass

    # pass to other methods
    def handle_starttag(self, tag, mth, attr):
        mth(tag)

    def handle_endtag(self, tag, mth):
        mth()

    # process the actual data
    def handle_data(self, data):
        # print self.lasttag, ':', data
        self.info[self.lasttag] = data

    # dont crash, just print
    def syntax_error(self, msg):
        # this means that there is a syntax problem in the xml, uh oh
        # print msg
        pass
